/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * transmit.c
 * Functions to handle sending datagrams
 * Copyright (C) 2006 Simon Newton
 */

#include "private.h"


/*
 * Build the sandnet advertisment packet
 */
int sandnet_tx_build_ad(node n) {
	int i;
	memset(&n->ad_temp, 0x00, sizeof(sandnet_ad_t)) ;


	n->ad_temp.op = htons(SANDNET_AD) ;

	memcpy(n->ad_temp.mac, n->state.hw_addr ,6) ;
	n->ad_temp.firmware = htonl(SANDNET_FIRMWARE_VER) ;

	for(i=0; i < SANDNET_MAX_PORTS; i++) {

		n->ad_temp.ports[i].proto = n->ports[i].proto ;
		n->ad_temp.ports[i].mode = n->ports[i].mode ;
		n->ad_temp.ports[i].term = n->ports[i].term ;
		n->ad_temp.ports[i].b = n->ports[i].b ;
		n->ad_temp.ports[i].param1 = n->ports[i].param1 ;
		n->ad_temp.ports[i].param2 = n->ports[i].param2 ;
	}

	n->ad_temp.nlen = strlen(n->state.name) ;
	strncpy((char*) n->ad_temp.name, n->state.name, SANDNET_NAME_LENGTH) ;

	n->ad_temp.magic3[0] = 0xc0;
	n->ad_temp.magic3[1] = 0xa8;
	n->ad_temp.magic3[2] = 0x01;
	n->ad_temp.magic3[3] = 0xa0;
	n->ad_temp.magic3[4] = 0x00;
	n->ad_temp.magic3[5] = 0xff;
	n->ad_temp.magic3[6] = 0xff;
	n->ad_temp.magic3[7] = 0xff;
	n->ad_temp.magic3[8] = 0x00;
	n->ad_temp.led =  n->state.led;
	n->ad_temp.magic4 = 0x01;

	return SANDNET_EOK ;

}


/*
 * Send a sandnet advertisment
 */
int sandnet_tx_send_ad(node n) {
	sandnet_packet_t ad ;
	memset(&ad, 0x00, sizeof(ad)) ;

	inet_aton("237.1.1.1", &ad.to ) ; 
	ad.length = sizeof(sandnet_ad_t) ;

	memcpy(&ad.data, &n->ad_temp, ad.length) ;

	return sandnet_net_ctl_send(n, &ad) ; 

}


/*
 * encode the dmx for sandnet
 *
 * @return the length used in the dst array
 */
int sandnet_tx_encode_dmx(uint8_t *src, int slen, uint8_t *dst, int dlen) {
	int i, j, l;
	int di = 0;

	for(i=0; i < slen && di < dlen; ) {

		// look ahead to see if this is a repeating value
		j = i+1 ;
		while(j < slen && src[i] == src[j] && j-i < 0x7f ) {
			j++ ;
		}

		// if the number of repeats is more than 2
		// don't encode only two repeats, 
		if(j-i > 2) {
			// if room left in dst buffer
			if(dlen -di > 1) {
				dst[di++] = (REPEAT_FLAG | (j-i)) ;
				dst[di++] = src[i] ;
			} else {
				// else return what we have done so far
				return di ;
			}
			i = j ;

		} else {
			// this value doesn't repeat more than twice
			// find out where the next repeat starts 

			// postcondition: j is one more than the last value we want to send
			for ( j = i+1 ; j < slen - 2 && j-i < 0x7f; j++) {

				// at the end  of the array
				if ( j == slen - 2 ) {
					j = slen;
					break ;
				}

				// if we're found a repeat of 3 or more stop here
				if ( src[j] == src[j+1] && src[j+1] == src[j+2] )
					break ;

			}

 			// if we have enough room left for all the values
			if(di+j-i < dlen) {
				dst[di++] = j-i ;
				memcpy(&dst[di], &src[i], j-i) ;
				di += j-i ;
				i += j-i ;
			
			// see how much data we can get in
			} else if(dlen - di > 1){
				l = dlen - di -1 ;
				dst[di++] = l ;
				memcpy(&dst[di], &src[i], l) ;
				di += l ;
				return di ;

			} else {
				return di ;
			}
		}	
	}
	return di ;
}



/*
 * Send some dmx data
 *
 */
int sandnet_tx_send_dmx(node n, uint8_t port, uint8_t grp, uint8_t uni, int length, uint8_t *data) {
	sandnet_packet_t p ;
	int ret, len ;
	
	len = min(length, SANDNET_DMX_LENGTH) ;
	memset(&p.data, 0x00, sizeof(p.data)) ;
	
	// set dst addr and length
	p.to.s_addr = n->state.data_addr.s_addr ;
	p.length = sizeof(sandnet_dmx_t) ;

	// setup the fields in the datagram
	p.data.dmx.op = htons(SANDNET_DMX) ;
	p.data.dmx.grp = grp;
	p.data.dmx.uni = uni;
	p.data.dmx.port = port ;
	memcpy(p.data.dmx.dmx, data, len) ;

	ret = sandnet_net_data_send(n, &p) ;

	return ret ;
}

